==> open-xchange-core_7.4.0.orig.tar.bz2: open-xchange-core-7.4.0/com.openexchange.java/src/com/openexchange/java/DirectIO.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.java;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.nio.ByteBuffer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * {@link DirectIO} - A utility class for direct byte buffers and I/O.
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class DirectIO {

    /**
     * Initializes a new {@link DirectIO}.
     */
    private DirectIO() {
        super();
    }

    private static volatile Method cleanerMethod;
    private static Method cleanerMethod() {
        Method m = cleanerMethod;
        if (null == m) {
            synchronized (DirectIO.class) {
                m = cleanerMethod;
                if (null == m) {
                    try {
                        m = ByteBuffer.class.getMethod("cleaner");
                        m.setAccessible(true);
                        cleanerMethod = m;
                    } catch (final Exception e) {
                        final Log log = LogFactory.getLog(DirectIO.class);
                        log.error("Couldn't initialze Java Reflection method for \"cleaner\".", e);
                    }
                }
            }
        }
        return m;
    }

    /**
     * DirectByteBuffers are garbage collected by using a phantom reference and a reference queue. Every once a while, the JVM checks the
     * reference queue and cleans the DirectByteBuffers. However, as this doesn't happen immediately after discarding all references to a
     * DirectByteBuffer, it's easy to OutOfMemoryError yourself using DirectByteBuffers. This function explicitly calls the Cleaner method
     * of a DirectByteBuffer.
     *
     * @param toBeDestroyed The DirectByteBuffer that will be "cleaned". Utilizes reflection.
     */
    public static void destroyDirectByteBuffer(final ByteBuffer toBeDestroyed) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, SecurityException, NoSuchMethodException {
        if (null == toBeDestroyed || !toBeDestroyed.isDirect()) {
            return;
        }

        final Method cleanerMethod = cleanerMethod();
        final Object cleaner = cleanerMethod.invoke(toBeDestroyed);
        final Method cleanMethod = cleaner.getClass().getMethod("clean");
        cleanMethod.setAccessible(true);
        cleanMethod.invoke(cleaner);
    }

}

==> open-xchange-core_7.4.2.orig.tar.bz2: open-xchange-core-7.4.2/com.openexchange.java/src/com/openexchange/java/DirectIO.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.java;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.nio.ByteBuffer;
import org.slf4j.Logger;

/**
 * {@link DirectIO} - A utility class for direct byte buffers and I/O.
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class DirectIO {

    /**
     * Initializes a new {@link DirectIO}.
     */
    private DirectIO() {
        super();
    }

    private static volatile Method cleanerMethod;
    private static Method cleanerMethod() {
        Method m = cleanerMethod;
        if (null == m) {
            synchronized (DirectIO.class) {
                m = cleanerMethod;
                if (null == m) {
                    try {
                        m = ByteBuffer.class.getMethod("cleaner");
                        m.setAccessible(true);
                        cleanerMethod = m;
                    } catch (final Exception e) {
                        final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(DirectIO.class);
                        log.error("Couldn't initialze Java Reflection method for \"cleaner\".", e);
                    }
                }
            }
        }
        return m;
    }

    /**
     * DirectByteBuffers are garbage collected by using a phantom reference and a reference queue. Every once a while, the JVM checks the
     * reference queue and cleans the DirectByteBuffers. However, as this doesn't happen immediately after discarding all references to a
     * DirectByteBuffer, it's easy to OutOfMemoryError yourself using DirectByteBuffers. This function explicitly calls the Cleaner method
     * of a DirectByteBuffer.
     *
     * @param toBeDestroyed The DirectByteBuffer that will be "cleaned". Utilizes reflection.
     */
    public static void destroyDirectByteBuffer(final ByteBuffer toBeDestroyed) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, SecurityException, NoSuchMethodException {
        if (null == toBeDestroyed || !toBeDestroyed.isDirect()) {
            return;
        }

        final Method cleanerMethod = cleanerMethod();
        final Object cleaner = cleanerMethod.invoke(toBeDestroyed);
        final Method cleanMethod = cleaner.getClass().getMethod("clean");
        cleanMethod.setAccessible(true);
        cleanMethod.invoke(cleaner);
    }

}
